﻿<CLSCompliant(True), Serializable()> _
Public Class CFolderHash : Inherits List(Of CFileHash)


#Region "Hashing"
    Private m_hash As Guid
    Public ReadOnly Property Hash As Guid
        Get
            Return m_hash
        End Get
    End Property
    Public ReadOnly Property Base64 As String
        Get
            Return CBinary.ToBase64(m_hash.ToByteArray)
        End Get
    End Property
#End Region

#Region "Ageing"
    Private m_created As DateTime = DateTime.Now
    Public ReadOnly Property Created As DateTime
        Get
            Return m_created
        End Get
    End Property
    Public ReadOnly Property Age As TimeSpan
        Get
            Return DateTime.Now.Subtract(Created)
        End Get
    End Property
#End Region


#Region "Constructor"
    Public Shared DEFAULT_EXCEPT_EXTENSIONS As String() = {".txt", ".config"}
    Public Sub New(folderPath As String, Optional exceptExtensions As String() = Nothing)
        Me.New(IO.Directory.GetFiles(folderPath), exceptExtensions)
    End Sub
    Public Sub New(files As String(), Optional exceptExtensions As String() = Nothing)
        MyBase.New(files.Length)

        If IsNothing(exceptExtensions) Then exceptExtensions = DEFAULT_EXCEPT_EXTENSIONS

        'Hash the files
        For Each i As String In files
            i = i.ToLower
            Dim skip As Boolean = False
            For Each j As String In exceptExtensions
                If i.EndsWith(j) Then
                    skip = True
                    Exit For
                End If
            Next
            If i.ToLower.Contains("vshost.") Then skip = True
            If Not skip Then Add(New CFileHash(i))
        Next

        'Hash the result (MD5)
        Dim list As New List(Of Byte)(Me.Count * 16)
        For Each i As CFileHash In Me
            list.AddRange(i.MD5.ToByteArray)
        Next
        m_hash = New Guid(CBinary.MD5(list.ToArray))
    End Sub
#End Region

#Region "Index On Name"
    <NonSerialized()> Private m_dict As Dictionary(Of String, CFileHash)
    Public ReadOnly Property Dict As Dictionary(Of String, CFileHash)
        Get
            If IsNothing(m_dict) OrElse m_dict.Count <> Me.Count Then
                m_dict = New Dictionary(Of String, CFileHash)(Me.Count)
                For Each i As CFileHash In Me
                    m_dict(i.Name) = i
                Next
            End If
            Return m_dict
        End Get
    End Property
#End Region

#Region "Diff Logic"
    Public Function ResolveDifferences(old As CFolderHash, folderPath As String) As List(Of CFileNameAndContent)
        Dim diff As New List(Of CFileNameAndContent)

        'Inserts, Updates
        For Each i As CFileHash In Me
            Dim j As CFileHash = Nothing
            If Not old.Dict.TryGetValue(i.Name, j) OrElse j.MD5 <> i.MD5 Then
                diff.Add(AddOrUpdate(i.Name, folderPath))
            End If
        Next
        For Each i As CFileHash In old
            If Not Me.Dict.ContainsKey(i.Name) Then diff.Add(Delete(i.Name))
        Next

        Return diff
    End Function
    Private Function AddOrUpdate(name As String, folderPath As String) As CFileNameAndContent
        Dim fnc As New CFileNameAndContent
        fnc.Name = name
        fnc.Content = IO.File.ReadAllBytes(folderPath & name)
        Return fnc
    End Function
    Private Function Delete(name As String) As CFileNameAndContent
        Dim fnc As New CFileNameAndContent
        fnc.Name = name
        fnc.Content = Nothing
        Return fnc
    End Function
#End Region

End Class
